<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2015 HUBzero Foundation, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-irods
 * @author    Rob Campbell <rcampbel@purdue.edu>
 * @copyright Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license   http://opensource.org/licenses/MIT MIT
 */

// No direct access
defined('_HZEXEC_') or die();

require_once PATH_APP . DS . 'plugins' . DS . 'filesystem' . DS . 'irods' . DS . 'prods'. DS . 'src' . DS . 'Prods.inc.php';
require_once PATH_APP . DS . 'plugins' . DS . 'filesystem' . DS . 'irods' . DS . 'prods'. DS . 'src' . DS . 'ProdsStreamer.class.php';

/**
 * Utility for iRODS access via prods (iRODS PHP API)
 */
class Prods extends Hubzero\Base\Object
{
	/**
	* Generate prods file object for a given hub file object
	*
	* @param   Hubzero\Filesystem\File  $file        The file to which the metadata pertains
	* @return  object     Prods file
	**/
	public static function getProdsFile(Hubzero\Filesystem\File $file)
	{
		$base   = $file->getAdapter()->getAdapter()->getBaseDir();
		$prodsf = Prods::prodsFileConnect($base . $file->path);

		return $prodsf;
	}

	/**
	* Generate prods file object for a given project file path
	*
	* @param   string  	$path Path to file incl. project (alias), excl. local mount path
	* @return  object	a ProdsFile object for given path
	**/
	public static function prodsFileConnect($path='')
	{
		$config  = Plugin::params('filesystem', 'irods');

		$account = new RODSAccount(
			$config->get('host')
			, intval($config->get('port'))
			, $config->get('user')
			, $config->get('pass')
			, $config->get('zone')
			, $config->get('resc')
		);

		$prefix  = '/' . $config->get('zone') . '/home/' . $config->get('user'); // NOTE: Hardcoded "home"

		$rodsfile = new ProdsFile($account, $prefix . '/' . $path);

		return $rodsfile;
	}
}
