<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2015 HUBzero Foundation, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-irods
 * @author    Rob Campbell <rcampbel@purdue.edu>
 * @copyright Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license   http://opensource.org/licenses/MIT MIT
 */

// NOTE This file was adapted from HUBzero core view annotate.php

// No direct access
defined('_HZEXEC_') or die();

// TODO Why don't following two lines work?
$this->css('irods');
$this->js('irods');
// TODO Remove temp code when lines above work
echo '<link rel="stylesheet" href="/app/plugins/metadata/irods/assets/css/irods.css" type="text/css" />';
echo '<script src="/app/plugins/metadata/irods/assets/js/irods.js" type="text/javascript"></script>';

Lang::load('plg_metadata_irods', PATH_APP . DS . 'plugins' . DS . 'metadata' . DS . 'irods');

// Directory path breadcrumbs
$bc    = \Components\Projects\Helpers\Html::buildFileBrowserCrumbs($this->subdir, $this->url, $parent, false);
$bcEnd = $this->item->isDir() ? '<span class="folder">' . $this->item->getName() . '</span>' : '<span class="file">' . $this->item->getName() . '</span>';
$lang  = $this->item->isDir() ? Lang::txt('PLG_METADATA_IRODS_FOLDER') : Lang::txt('PLG_METADATA_IRODS_FILE');

$dubCore = [
	'description',
	'title',
	'subject',
	'contributor',
	'publisher',
	'date',
	'identifier',
	'format',
	'type',
	'source',
	'creator',
	'language'
];

$geoFields = [
	'projection',
	'northlimit',
	'southlimit',
	'eastlimit',
	'westlimit',
	'latmin',
	'latmax',
	'lonmin',
	'lonmax',
	'xsize',
	'ysize'
];
?>

<div id="abox-content">
	<h3>
		<?php echo Lang::txt('PLG_METADATA_IRODS_TITLE') . ' ' . $lang . ' ' . $bc . ' ' . $bcEnd; ?>
	</h3>
	<?php if ($this->getError()) : ?>
		<p class="witherror"><?php $this->getError(); ?></p>
	<?php else : ?>

		<div id="scroll-area">

		<form id="hubForm-ajax" method="post" action="<?php echo Route::url($this->url); ?>">
			<fieldset>
				<input type="hidden" name="subdir" value="<?php echo $this->subdir; ?>" />
				<input type="hidden" name="action" value="annotateit" />
				<input type="hidden" name="item" value="<?php echo $this->item->getName(); ?>" />

				<ul id="metadata-entries">
					<?php $i = 0; ?>

					<!-- Dublin Core: display values and remove from metadata array -->
					<?php echo '<h4 onclick="openClose('."'core','"
						. Lang::txt('PLG_METADATA_IRODS_SHOW') .  "','"
						. Lang::txt('PLG_METADATA_IRODS_HIDE') . "')" . '">'
						. Lang::txt('PLG_METADATA_IRODS_LABEL_CORE')
						. '<span class="core-sign show-hide-label">'
							. Lang::txt('PLG_METADATA_IRODS_HIDE')
						. '</span></h4>';
					?>
					<div class="core-area">
						<?php foreach ($dubCore as $element) : ?>
							<li>
								<div class="entry key-value-pair" data-idx="<?php echo $i; ?>">
									<div class="entry-label">
										<input type="text" name="key[<?php echo $i; ?>]" maxlength="250" value="<?php echo $element; ?>" readonly />
									</div>
									<div class="separator">:</div>
									<div class="entry-value">
										<?php
											$use_value = '';
											$use_units = '';
											if (isset($this->metadata[$element]))
											{
												if ($this->metadata[$element]['units'] == '')
												{
													$use_value = $this->metadata[$element]['value'];
													$use_units = $this->metadata[$element]['units'];
													unset($this->metadata[$element]);
												}
											}
										?>
										<input type="text"   name="value[<?php echo $i; ?>]" value="<?php echo $use_value; ?>" />
										<input type="hidden" name="units[<?php echo $i; ?>]" value="<?php echo $use_units; ?>" />
									</div>
								</div>
							</li>
							<?php $i++; ?>
						<?php endforeach; ?>
					</div>

					<!-- Geographic: display values and remove from metadata array -->
					<?php echo '<h4 onclick="openClose('."'geo','"
						. Lang::txt('PLG_METADATA_IRODS_SHOW') . "','"
						. Lang::txt('PLG_METADATA_IRODS_HIDE') . "')". '">Geographic Coverage<span class="geo-sign show-hide-label">'
						. Lang::txt('PLG_METADATA_IRODS_HIDE')
						. '</span></h4>';
					?>
					<div class="geo-area">
						<?php foreach ($geoFields as $element) : ?>
							<?php
								$use_value = '';
								$use_units = '';
								if (isset($this->metadata[$element]))
								{
									if ($this->metadata[$element]['units'] == '')
									{
										$use_value = $this->metadata[$element]['value'];
										$use_units = $this->metadata[$element]['units'];
										unset($this->metadata[$element]);
									}
								}
							?>
							<li>
								<div class="entry key-value-pair" data-idx="<?php echo $i; ?>">
									<div class="entry-label">
										<input type="text" name="key[<?php echo $i; ?>]" maxlength="250" value="<?php echo $element; ?>" readonly />
									</div>
									<div class="separator">:</div>
									<div class="entry-value">
										<input type="text"   name="value[<?php echo $i; ?>]" value="<?php echo $use_value; ?>"  readonly />
										<input type="hidden" name="units[<?php echo $i; ?>]" value="<?php echo $use_units; ?>" />
									</div>
								</div>
							</li>
							<?php $i++; ?>
						<?php endforeach; ?>
					</div>

					<!-- User Defined (display remaining fields in metadata array) -->
					<?php echo '<h4 onclick="openClose('."'external','"
						. Lang::txt('PLG_METADATA_IRODS_SHOW') . "','"
						. Lang::txt('PLG_METADATA_IRODS_HIDE') . "')".'">User-Defined Metadata<span class="external-sign show-hide-label">'
						. Lang::txt('PLG_METADATA_IRODS_HIDE') . '</span></h4>';
					?>
					<div class="external-area">
						<?php foreach ($this->metadata as $name => $item) : ?>
							<?php if (in_array($name,$this->hideFields)) continue; ?>
							<li>
								<div class="entry key-value-pair" data-idx="<?php echo $i; ?>">
									<div class="entry-label">
										<input type="text" name="key[<?php echo $i; ?>]" maxlength="250" value="<?php echo $name; ?>" />
									</div>
									<div class="separator">:</div>
									<div class="entry-value">
										<input type="text"   name="value[<?php echo $i; ?>]" value="<?php echo $item['value']; ?>" />
										<input type="hidden" name="units[<?php echo $i; ?>]" value="<?php echo $item['units']; ?>" />
									</div>
								</div>
							</li>
							<?php $i++; ?>
						<?php endforeach; ?>

						<!-- New entries -->
						<li>
							<div class="entry key-value-pair" data-idx="<?php echo $i; ?>">
								<div class="entry-label">
									<input type="text" name="key[<?php echo $i; ?>]" maxlength="250" value="" />
								</div>
								<div class="separator">:</div>
								<div class="entry-value">
									<input type="text"   name="value[<?php echo $i; ?>]" value="" />
									<input type="hidden" name="units[<?php echo $i; ?>]" value="external" />
								</div>
							</div>
						</li>
					</div>
				</ul>

				<div class="btn icon-add add-new-annotation">
					<?php echo Lang::txt('PLG_METADATA_IRODS_ADD'); ?>
				</div>

				<div class="buttons">
					<input type="submit" class="btn" value="<?php echo Lang::txt('PLG_METADATA_IRODS_SAVE'); ?>" />
					<input type="reset" class="btn btn-cancel" id="cancel-action" value="<?php echo Lang::txt('PLG_METADATA_IRODS_CANCEL'); ?>" />
				</div>
			</fieldset>
		</form>

		</div>

	<?php endif; ?>
</div>
