<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2015 HUBzero Foundation, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-cms
 * @copyright Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license   http://opensource.org/licenses/MIT MIT
 */

// No direct access
defined('_HZEXEC_') or die();

use Components\Projects\Models\Orm\Project;
use Components\Projects\Models\Orm\Connection;
use Components\Projects\Models\Orm\Provider;
use Components\Projects\Models\Orm\Owner;

require_once Component::path('com_projects') . '/models/orm/owner.php';
require_once Component::path('com_projects') . '/models/orm/connection.php';
require_once Component::path('com_projects') . '/models/orm/provider.php';
require_once Component::path('com_projects') . '/models/orm/project.php';
require_once Component::path('com_projects') . '/models/project.php';


/**
 * Search groups
 */
class plgSearchIrods extends \Hubzero\Plugin\Plugin
{
	/**
	 * onGetTypes - Announces the available hubtype
	 * 
	 * @param mixed $type 
	 * @access public
	 * @return void
	 */
	public function onGetTypes($type = null)
	{
		// The name of the hubtype
		$hubtype = 'project-file';

		if (isset($type) && $type == $hubtype)
		{
			return $hubtype;
		}
		elseif (!isset($type))
		{
			return $hubtype;
		}
	}

	/**
	 * onAddPermissionSet - Added customized permission restrictions
	 * 
	 * @access public
	 * @return string SOLR restrictive filter query
	 */
	public function onAddPermissionSet()
	{
		if (!User::isGuest())
		{
			// Get all of the projects the user belongs to
			$rosterEntries = Owner::all()->whereEquals('userid', User::get('id'))->rows();

			// Initalize the empty set
			$string = ' OR (hubtype:project-file AND owner:(';
			$i = 0;

			// Make a string of the project aliases
			foreach ($rosterEntries as $entry)
			{
				$project = $entry->project()->row()->get('alias');
				$string .= $project;

				// Conditionally append OR 
				if ($i < $rosterEntries->count() - 1)
				{
					$string .= ' OR ';
				}
				elseif ($i == $rosterEntries->count() - 1)
				{
					$string .= '))';
				}

				// Advance iterator
				$i++;
			}
			return $string;
		}
	}

	/**
	 * onFormatResult  - Allows view override for result
	 * 
	 * @param mixed $type 
	 * @param mixed $result 
	 * @param mixed $terms 
	 * @param mixed $highlightOptions 
	 * @access public
	 * @return void
	 */
	public function onFormatResult($type, &$result, $terms, $highlightOptions)
	{
		if ($type == 'project-file')
		{
			
			/**
			 * This is an example of a View being returned by the search result.
			 * The view is displayed in the display.php view of com_search.
			 **/

			// The view is located in plugins/search/irods/views/result/tmpl/default.php
			$customView = new \Hubzero\Plugin\View([
				'folder' => 'search',
				'element' => 'irods',
				'name' => 'result',
				'layout' => 'default'
			]);

                	$config  = Plugin::params('filesystem', 'irods');

			$identifier = $result['id'];

                	$prefix  = '/' . $config->get('zone') . '/home/' . $config->get('user') . '/'; // NOTE: Hardcoded "home"

			if(strpos($identifier,$prefix) !== FALSE)
			{
				$project_path = substr($identifier,strlen($prefix));
				if(strpos($project_path,'/') !== FALSE)
				{
					$project_alias = substr($project_path,0,strpos($project_path,'/'));
					$lastslash = strrpos($project_path,'/');
					$firstslash = strpos($project_path,'/');
					if($firstslash == $lastslash) //file is at root directory
					{
						$relative_path = FALSE;
					}
					else
					{
						$relative_path = substr($project_path,$firstslash+1,($lastslash - $firstslash - 1));
					}

					//get connection ID for irods adapter for this project
					//also set project title for display in result
					$cid = FALSE;
					try
					{
						$project = new \Components\Projects\Models\Project($project_alias);
						$result['project'] = $project->get('title');
						$connections = Project::oneOrFail($project->get('id'))->connections()->thatICanView();
						foreach ($connections as $connection)
						{
							if($connection->provider->alias == 'irods')
							{
								$cid = $connection->get('id');
							}
						}
						if($cid)
						{
							$result['url'] = Route::url('projects/'.$project_alias.'/files/browse?connection='.$cid);
							if($relative_path)
							{
								$result['url'] = $result['url'].'&subdir='.$relative_path;
							}
						}
						else
						{
							$result['url'] = '';
						}	
					}
					catch (Exception $e)
					{
						$result['url'] = '';
					}
				}
				else //couldn't identify project
				{
					$result['url'] = '';
				}
			}
			else
			{
				$result['url'] = '';
			}

			$lastslash = strrpos($identifier,'/');
			if($lastslash !== FALSE)
			{
				$filename = substr($identifier,$lastslash+1);
			}
			else
			{
				$filename = $identifier;
			}
		
			//title is the primary search result, we set it to the filename
			//but use title in the snippet field	
			$result['title'] = $filename;

			// The "fulltext" portion of snippet
			if(array_key_exists('description',$result))
			{
				$result['snippet'] = $result['description'];
			}
			else if(array_key_exists('title',$result) && ($result['title'] !== $filename))
			{
				$result['snippet'] = $result['title'];
			}
			else
			{
				$result['snippet'] = '';
			}

			// Push the result to the view
			$customView->result = $result;

			return $customView;
		}
	}
}
