/**
 * @package     hubzero-cms
 * @file        templates/geo/html/plg_projects_files/connected/fileupload.flow.js
 * @copyright   Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license     http://opensource.org/licenses/MIT MIT
 */

var r = new Flow({
	target:$("#ajax-uploader").attr("data-action"),
	chunkSize:2*1024*1024,
	forceChunkSize: true,
	preprocess: function(chunk)
		{
        	   var func   = (chunk.fileObj.file.slice ? 'slice' : 
						(chunk.fileObj.file.mozSlice ? 'mozSlice' : 
							(chunk.fileObj.file.webkitSlice ? 'webkitSlice' : 'slice'))),
	               bytes  = chunk.fileObj.file[func](chunk.startByte,chunk.endByte),
                       spark = null,
                       fileReader = new FileReader();

                       //callback waiting for data to be read and hash to be computed
                       fileReader.onloadend = function(e) {
                         spark = SparkMD5.ArrayBuffer.hash(e.target.result);
			 //set this property to be retrieved later when constructing the request
			 chunk.spark = spark;
			 //indicate that preprocessing is done
			 chunk.preprocessFinished();
			};

		   fileReader.readAsArrayBuffer(bytes);
		},
	//additional request parameters
	//send the chunk hash 
	query: function (flowFile, flowChunk, isTest) 
		{
		  if(isTest)
		  {
		     return {'flowChunkHash':flowChunk.spark};
		  }
		  return {};
		}
        });

var fileTemplate = '<li>' +
               	    '<span class="qq-upload-icon"></span>' +
                    '<span class="qq-upload-file"></span>' +
                    '<span class="qq-upload-name"></span>' +
                    '<span class="qq-upload-status"></span>' +
                    '<a class="qq-upload-cancel" href="#">Cancel</a>' +
                    '<span class="qq-upload-ext"></span>' +
                    '<span class="qq-upload-size"></span>' +
                    '<span class="qq-upload-spinner"></span>' +
                    '<span class="qq-upload-error"></span>' +
                  '</li>';

function getCss(ext)
{
    var type = 'file';
    
    // Compressed file extensions
    var tar = {
	'gz'  	: 1,
	'7z'    : 1,
	'zip' 	: 1,
	'zipx' 	: 1,
	'sit' 	: 1,
	'sitx' 	: 1,
	'rar' 	: 1
    };
    
    // Video file extensions
    var video = {
	'avi'  	: 1,
	'mpeg' 	: 1,
	'mov' 	: 1,
	'mp4'  	: 1,
	'mpg'  	: 1,
	'rm'  	: 1,
	'ogg'  	: 1,
	'wmv'  	: 1
    };
    
    // Image file extensions
    var image = {
	'bmp'  : 1,
	'jpeg' : 1,
	'jpg'  : 1,
	'jpe'  : 1,
	'gif'  : 1,
	'png'  : 1,
	'tif'  : 1,
	'tiff' : 1
    };
    
    // Text file extensions
    var text = {
	'tex'  : 1,
	'txt'  : 1,
	'rtf'  : 1,
	'php'  : 1,
	'css'  : 1,
	'html' : 1,
	'csv'  : 1
    };
    
    // Office file extensions
    var office = {
	'xls'   : 1,
	'xlsx'  : 1,
	'doc'   : 1,
	'docx'  : 1,
	'ppt'   : 1,
	'pptx'  : 1
    };
    
    if (ext == 'pdf')
	{
	    type = 'pdf';
	}
    else if (office[ext]) {
	type = 'office';
    }
    else if (tar[ext]) {
	type = 'archive';
    }
    else if (video[ext]) {
	type = 'video';
    }
    else if (image[ext]) {
	type = 'image';
    }
    else if (text[ext]) {
	type = 'text';
    }
    
    var cssClass = 'type-' + type;
    return cssClass;
}

function formatSize(bytes){
    var i = -1;                                    
    do {
	bytes = bytes / 1024;
	i++;  
    } while (bytes > 99);
    
    if (bytes == 0)
	{
	    return '0b';
	}
    
    return Math.max(bytes, 0.1).toFixed(1) + ['kB', 'MB', 'GB', 'TB', 'PB', 'EB'][i];          
}

function findFileElem(file) {
    var fName = file.name;
    var fileElem = false;
    $("#fileList li").each(function(index,element) {
	    var $fileItem = $(element).find(".qq-upload-file");
	    if($fileItem.text() == fName)
		{
		    fileElem = $(element);
		    return false;
		}
	});

    return fileElem;
}


r.assignBrowse(document.getElementById('browseFiles'));
r.assignDrop(document.getElementById('browseFiles'));

r.on('fileStart',function(file){
	var d = new Date();
	var time = d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds();
    });

r.on('fileSuccess', function(file){
	var d = new Date();
	var time = d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds();

	r.removeFile(file);

	if(r.files.length > 0)
	    r.files[0].resume();

	var fName = file.name;

	//update list entry
	
	var fileElem = findFileElem(file);

	if(fileElem)
	    {
		var fileCancel = fileElem.find(".qq-upload-cancel");
		var fileStatus = fileElem.find(".qq-upload-status");

		fileCancel.html('');
		fileStatus.text('uploaded');
		fileStatus.addClass('qq-upload-success');
	    }

    });
r.on('fileProgress', function(file){

	$('#progressBar').val(Math.floor(file.progress() * 100));

        var fileElem = findFileElem(file);

	if(fileElem)
	    {
		var can = fileElem.find(".qq-upload-cancel");
		can.html('');
	
		var elExt  = fileElem.find(".qq-upload-ext");
		elExt.html('');
	
		var size = fileElem.find(".qq-upload-size");
		size.css('display','inline');
	
		var status = fileElem.find(".qq-upload-status");
		status.html('getting data');

		status.addClass('qq-upload-processing');
	
		var text; 
		var perc = Math.round(file.progress() * 100);
		text = file.size > 0 ? perc + '% from ' + formatSize(file.size) : 'loaded';
	
		if (perc >= 99) {   
		    text = formatSize(file.size);
		    status.html('processing');
		}          

		size.text(text);
	    }

    });
r.on('fileAdded', function(file, event){

	var $fileItem = $.parseHTML(fileTemplate);
	var $listElement = $("#fileList");

	//create and add a new li element based on the fileTemplate
	$listElement.append($fileItem);
	$fileItem = $listElement.children("li:last");

	//populate the filename, size and extension spans
	var fileName = file.name;
	var fSize = formatSize(file.size);
	var ext = fileName.split('.').pop().toLowerCase();
	if (ext.length > 4) { ext = ''; }

        var $fileElement = $fileItem.find(".qq-upload-file");
        var $elSize  = $fileItem.find(".qq-upload-size");
	var $elExt = $fileItem.find(".qq-upload-ext");
	var $elIcon = $fileItem.find(".qq-upload-icon");

	//setup onClick for cancel links in upload file list
	$(".qq-upload-cancel").on('click', function(e) {
		e.preventDefault();
		
		var $fileItem = $(this).parent().find(".qq-upload-file");
		
		//also delete from Flow file array
		 for (var i=0; i< r.files.length; i++)
		     {
			 if(r.files[i].name == $fileItem.text())
			     {
				 r.files.splice(i,1);
		 	    }
		     }
		 $(this).parent().get(0).remove();
		 reportChange();
	});

	$fileElement.text(fileName);
	$elSize.text(fSize);
	$elExt.text(ext);
	
	var css = getCss(ext);
		
	if ($elIcon)
	    {
		$elIcon.addClass(css);
	    }

	reportChange();

    });
r.on('filesAdded', function(array){
    });
r.on('fileRetry', function(file){
    });
r.on('fileError', function(file, message){
    });
r.on('uploadStart', function(){

	$('#f-upload').val('Upload started. Please wait');
	$('#f-upload').addClass('started');
	
	if ($('#cancel-action'))
	    {
		$('#cancel-action').remove();
	    }
	
	$('#upload-instruct').html('Uploading selected file(s). Please do not close this window.');   
	
    });
r.on('complete', function(){
	var form = $('#hubForm-ajax');
	if (form.length)
	    {			
		window.location.replace(form.attr('action'));
	    }
    });
r.on('progress', function(){
    });
r.on('error', function(message, file){
    });
r.on('pause', function(){
    });
r.on('cancel', function(){
    });

function checkButton()
{
    if ($('#f-upload').length == 0)
	{
	    return false;
	}
    
    if ($("#fileList li").length == 0)
	{
	    $('#f-upload').addClass('disabled');
	}
    else if ($('#f-upload').hasClass('disabled'))
	{
	    $('#f-upload').removeClass('disabled');
	}
}

function addUpSize() 
{	
    var cSize = 0;
    if (r.files.length > 0)
	{
	    for (var i=0; i< r.files.length; i++)
		{
		    var fSize = r.files[i].size;
		    if(fSize)
			{
			    cSize = cSize + fSize;
			}
		}
	}
    return cSize;
}

function getAvailSpace(cSize)
{
    var avail = 0;
    if ($('#avail').length)
	{
	    avail = $('#avail').val();
	}
    
    avail = avail - cSize;
    avail = avail < 0 ? 0 : avail;
    return avail;
}
    
function showTotals(cSize)
{
    if (cSize)
	{
	    if ($('#upload-csize').length)
		{
		    $('#upload-csize').empty();
		    $('#upload-csize').css('display', 'none');	
		}
	    
	    var avail = getAvailSpace(cSize);
	    
	    var availClass = avail < 100 ? 'urgency' : 'green';
	    avail = avail ? formatSize(avail) : 0;
	    
	    $('#upload-csize').css('display', 'block');	
	    $('#upload-csize').append('<ul><li>' +	
				      '<span id="csize-avail">Available: <span class="prominent ' + availClass + '">' + avail + '</span></span>' +					
				      '<span id="csize">Total: <span class="prominent">' + formatSize(cSize) + '</span></span>' +
				      '</li></ul>');				
	}
    else if ($('#upload-csize').length)
	{
	    $('#upload-csize').empty();
	    $('#upload-csize').css('display', 'none');	
	}
}

function reportChange()
{
    // Check total size against quota
    var cSize = addUpSize();
    
    // Show total size
    showTotals(cSize);	
    
    // Enable submit button?
    checkButton();	
}
    
function initialize() {
    if ($('#f-upload').length)
	{
	    $('#f-upload').addClass('btnaction');
	    $('#f-upload').addClass('disabled');
	    
	    $('#f-upload').on('click', function(e) {
		    e.preventDefault();
		    //automatically handles all files in its files[] array
		    if ($('#f-upload').hasClass('started'))
			{
			    return false;
			}
		
		    //pause all except the first one
		    //this is so that the simultaneousUpload only picks up
		    //chunks from one file at a time
		    for (var i=1 ; i< r.files.length; i++)
		     {
			 r.files[i].pause();

		 	 //update list entry to set status to 'waiting'
	
			 var fileElem = findFileElem(r.files[i]);

			 if(fileElem)
	    		 {
				var fileCancel = fileElem.find(".qq-upload-cancel");
				var fileStatus = fileElem.find(".qq-upload-status");

				fileCancel.html('');
				fileStatus.text('waiting');
		     	 }
		    }
		    r.upload();
		});
	}
}

jQuery(document).ready(function($){
	initialize();
});
